/**
 * Copyright (c) 2008-2012 EBM WebSourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the New BSD License (3-clause license).
 *
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the New BSD License (3-clause license)
 * for more details.
 *
 * You should have received a copy of the New BSD License (3-clause license)
 * along with this program/library; If not, see http://directory.fsf.org/wiki/License:BSD_3Clause/
 * for the New BSD License (3-clause license).
 */
 
package org.ow2.easywsdl.extensions.wsdl4complexwsdl.impl.inout;

import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.ow2.easywsdl.extensions.complexwsdl.ImportedDocuments;
import org.ow2.easywsdl.extensions.complexwsdl.ObjectFactory;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.Description;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlException;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlWriter;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.impl.DescriptionImpl;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.impl.ImportedDocumentsImpl;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.impl.WSDL4ComplexWsdlJAXBContext;
import org.ow2.easywsdl.schema.api.XmlException;
import org.ow2.easywsdl.wsdl.WSDLFactory;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.easywsdl.wsdl.api.WSDLWriter;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * @author Nicolas Salatge - EBM WebSourcing
 */
public class WSDL4ComplexWsdlWriterImpl implements WSDL4ComplexWsdlWriter {

	private WSDLWriter writer = null;


	private DocumentBuilderFactory builder = null;

	public WSDL4ComplexWsdlWriterImpl() throws WSDL4ComplexWsdlException {
		try {
			this.writer = WSDLFactory.newInstance().newWSDLWriter();

			builder = DocumentBuilderFactory.newInstance();
			builder.setNamespaceAware(true);

		} catch (final WSDLException e) {
			throw new WSDL4ComplexWsdlException(e);
		} 
	}

    @Override
    public Document getDocument(final org.ow2.easywsdl.wsdl.api.Description wsdlDef) throws WSDL4ComplexWsdlException {
		Document res = null;
		try {
            org.ow2.easywsdl.wsdl.api.Description desc;
            if (wsdlDef instanceof Description) {
                desc = ((DescriptionImpl) wsdlDef).getFirstDescription();

                org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.ImportedDocuments imptDocs = ((Description) wsdlDef)
                        .getImportedDocuments();

                if ((imptDocs != null) && (!imptDocs.getDocuments().isEmpty())
                        && !(existImportedDocumentInOthersElements(desc))) {
                    desc.addOtherElements(this.convertImportedDocuments2Element(
                            (ImportedDocuments) ((ImportedDocumentsImpl) imptDocs).getModel()));
                }
            } else {
                desc = wsdlDef;
			}
			
			res = this.writer.getDocument(desc);
			
		} catch (final WSDLException e) {
			throw new WSDL4ComplexWsdlException(e);
		} catch (XmlException e) {
			throw new WSDL4ComplexWsdlException(e);
		}
		return res;
	}

	private boolean existImportedDocumentInOthersElements(
			org.ow2.easywsdl.wsdl.api.Description desc) throws XmlException {
		boolean res = false;
		for(Element elmt: desc.getOtherElements()) {
			if(elmt.getLocalName().equals("importedDocuments")) {
				res = true;
				break;
			}
		}
		return res;
	}

    @Override
	public boolean getFeature(final String name) throws IllegalArgumentException {
		return this.writer.getFeature(name);
	}

    @Override
	public void setFeature(final String name, final boolean value) throws IllegalArgumentException {
		this.writer.setFeature(name, value);
	}

    @Override
    public String writeWSDL4ComplexWsdl(final org.ow2.easywsdl.wsdl.api.Description wsdlDef)
            throws WSDL4ComplexWsdlException {
		String res = null;
		try {
            org.ow2.easywsdl.wsdl.api.Description desc;
            if (wsdlDef instanceof Description) {
                desc = ((DescriptionImpl) wsdlDef).getFirstDescription();
                org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.ImportedDocuments imptDocs = ((Description) wsdlDef)
                        .getImportedDocuments();
                if ((imptDocs != null) && (!imptDocs.getDocuments().isEmpty())) {
                    desc.addOtherElements(this.convertImportedDocuments2Element(
                            (ImportedDocuments) ((ImportedDocumentsImpl) imptDocs).getModel()));
                }
            } else {
                desc = wsdlDef;
            }
			res = this.writer.writeWSDL(desc);
		} catch (XmlException e) {
			throw new WSDL4ComplexWsdlException(e);
		}
		return res;
	}

	//	public JAXBElement<ImportedDocuments> convertImportedDocuments2JAXBElement(final ImportedDocuments imptDocs)
	//	throws WSDLException {
	//		ObjectFactory factory = new ObjectFactory();
	//		return factory.createImportedDocuments(imptDocs);
	//	}

    private Element convertImportedDocuments2Element(final ImportedDocuments imptDocs) throws WSDLException {

		Document doc = null;
		try {
			doc = builder.newDocumentBuilder().newDocument();


			ObjectFactory factory = new ObjectFactory();
			JAXBElement<ImportedDocuments>  jaxbImptDoc = factory.createImportedDocuments(imptDocs);

			WSDL4ComplexWsdlJAXBContext.getJaxbContext().createMarshaller().marshal(jaxbImptDoc, doc);
		} catch (final JAXBException e) {
			throw new WSDLException(
					"Failed to build Java bindings from WSDL descriptor XML document", e);
		} catch (ParserConfigurationException e) {
			throw new WSDLException(
					"Failed to build Java bindings from WSDL descriptor XML document", e);
		}
		return doc.getDocumentElement();
	}
}
