/**
 * Copyright (c) 2008-2012 EBM WebSourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the New BSD License (3-clause license).
 *
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the New BSD License (3-clause license)
 * for more details.
 *
 * You should have received a copy of the New BSD License (3-clause license)
 * along with this program/library; If not, see http://directory.fsf.org/wiki/License:BSD_3Clause/
 * for the New BSD License (3-clause license).
 */
 
package org.ow2.easywsdl.extensions.wsdl4complexwsdl.test;

import java.io.File;
import java.io.IOException;
import java.net.URISyntaxException;
import java.net.URL;

import org.junit.Assert;
import junit.framework.TestCase;

import org.ow2.easywsdl.extensions.wsdl4complexwsdl.WSDL4ComplexWsdlFactory;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.Description;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlReader;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.w3c.dom.Document;

/**
 * Test for the notifications
 * @author Frederic Gardes - EBM WebSourcing
 */
public class WSDL4ComplexWsdlForNotificationTest extends TestCase {

	private WSDL4ComplexWsdlReader reader = null;
	
	/*
	 * (non-Javadoc)
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		super.setUp();
		reader = WSDL4ComplexWsdlFactory.newInstance().newWSDLReader();
	}

	public void testReadWSDL4ComplexWSDLNotificationConsumer() throws URISyntaxException, WSDLException, IOException {
	    final URL wsdlURL = new File("./src/test/resources/descriptors/notification/WS-NotificationConsumer.wsdl").toURI().toURL();
        Description wsdlDescription = reader.read(wsdlURL);
		// unify all import and include
		//Document doc = WSDL4ComplexWsdlFactory.newInstance().newWSDLWriter().getDocument(wsdlDescription);
		//System.out.println("[1] -- \n\n" + XMLPrettyPrinter.prettyPrint(doc));
		wsdlDescription.addImportedDocumentsInWsdl();
		//doc = WSDL4ComplexWsdlFactory.newInstance().newWSDLWriter().getDocument(wsdlDescription);
		//System.out.println("\n[2] -- \n\n" + XMLPrettyPrinter.prettyPrint(doc));
		wsdlDescription.deleteImportedDocumentsInWsdl();
		//doc = WSDL4ComplexWsdlFactory.newInstance().newWSDLWriter().getDocument(wsdlDescription);
		//System.out.println("\n[3] -- \n\n" + XMLPrettyPrinter.prettyPrint(doc));
		// assert on description
		Assert.assertNotNull(wsdlDescription);
	}

	public void testReadWSDL4ComplexWSDLNotificationProducer() throws URISyntaxException, WSDLException, IOException {
	    final URL wsdlURL = Thread.currentThread().getContextClassLoader().getResource("descriptors/notification/WS-NotificationProducer.wsdl");
        Description wsdlDescription = reader.read(wsdlURL);
		// unify all import and include
		wsdlDescription.addImportedDocumentsInWsdl();
		// assert on description
		Assert.assertNotNull(wsdlDescription);
	}

	public void testReadWSDL4ComplexWSDLNotificationBroker() throws URISyntaxException, WSDLException, IOException {
	    final URL wsdlURL = Thread.currentThread().getContextClassLoader().getResource("descriptors/notification/WS-NotificationBroker.wsdl");
        Description wsdlDescription = reader.read(wsdlURL);
		// unify all import and include
		
		wsdlDescription.addImportedDocumentsInWsdl();
		// assert on description
		Assert.assertNotNull(wsdlDescription);
	}

//	public void testWriteWSDL4ComplexWSDLNotificationConsumer() throws URISyntaxException, WSDLException {
//		Description wsdlDescription = reader.readWSDL4ComplexWSDL(new URI("./src/test/resources/descriptors/notification/WS-NotificationConsumer.wsdl"));
//		// unify all import and include
//		wsdlDescription.addImportedDocumentsInWsdl();
//		// assert on description
//		assertNotNull(wsdlDescription);
//		Document document = WSDL4ComplexWsdlFactory.newInstance().newWSDLWriter().getDocument(wsdlDescription);
//		assertNotNull(document);
//	}
//
//	public void testWriteWSDL4ComplexWSDLNotificationProducer() throws URISyntaxException, WSDLException {
//		Description wsdlDescription = reader.readWSDL4ComplexWSDL(new URI("./src/test/resources/descriptors/notification/WS-NotificationProducer.wsdl"));
//		// unify all import and include
//		wsdlDescription.addImportedDocumentsInWsdl();
//		// assert on description
//		assertNotNull(wsdlDescription);
//		Document document = WSDL4ComplexWsdlFactory.newInstance().newWSDLWriter().getDocument(wsdlDescription);
//		assertNotNull(document);
//	}

	public void testWriteWSDL4ComplexWSDLNotificationBroker() throws URISyntaxException, WSDLException, IOException {
	    final URL wsdlURL = Thread.currentThread().getContextClassLoader().getResource("descriptors/notification/WS-NotificationBroker.wsdl");
        Description wsdlDescription = reader.read(wsdlURL);
        // unify all import and include
		wsdlDescription.addImportedDocumentsInWsdl();
		// assert on description
		assertNotNull(wsdlDescription);
		Document document = WSDL4ComplexWsdlFactory.newInstance().newWSDLWriter().getDocument(wsdlDescription);
		assertNotNull(document);
	}
	
//	public void testReadOnWrittenWSDL4ComplexWSDLNotificationConsumer() throws URISyntaxException, WSDLException {
//		Description wsdlDescription = reader.readWSDL4ComplexWSDL(new URI("./src/test/resources/descriptors/notification/WS-NotificationConsumer.wsdl"));
//		// unify all import and include
//		wsdlDescription.addImportedDocumentsInWsdl();
//		// assert on description
//		assertNotNull(wsdlDescription);
//		Document document = WSDL4ComplexWsdlFactory.newInstance().newWSDLWriter().getDocument(wsdlDescription);
//		assertNotNull(document);
//		Description wsdlDescription2 = reader.readWSDL4ComplexWSDL(document);
//		assertNotNull(wsdlDescription2);
//	}
//
//	public void testReadOnWrittenWSDL4ComplexWSDLNotificationProducer() throws URISyntaxException, WSDLException {
//		Description wsdlDescription = reader.readWSDL4ComplexWSDL(new URI("./src/test/resources/descriptors/notification/WS-NotificationProducer.wsdl"));
//		// unify all import and include
//		wsdlDescription.addImportedDocumentsInWsdl();
//		// assert on description
//		assertNotNull(wsdlDescription);
//		Document document = WSDL4ComplexWsdlFactory.newInstance().newWSDLWriter().getDocument(wsdlDescription);
//		assertNotNull(document);
//		Description wsdlDescription2 = reader.readWSDL4ComplexWSDL(document);
//		assertNotNull(wsdlDescription2);
//	}

	public void testReadOnWrittenWSDL4ComplexWSDLNotificationBroker() throws URISyntaxException, WSDLException, IOException {
	    final URL wsdlURL = Thread.currentThread().getContextClassLoader().getResource("descriptors/notification/WS-NotificationBroker.wsdl");
        Description wsdlDescription = reader.read(wsdlURL);
        // unify all import and include
		wsdlDescription.addImportedDocumentsInWsdl();
		// assert on description
		assertNotNull(wsdlDescription);
		Document document = WSDL4ComplexWsdlFactory.newInstance().newWSDLWriter().getDocument(wsdlDescription);
		document.setDocumentURI(wsdlURL.toString());
		assertNotNull(document);
		Description wsdlDescription2 = reader.read(document);
		assertNotNull(wsdlDescription2);
	}

}
